import graphene
from pyquery import PyQuery as pq


class Node(graphene.Interface):
    '''A Node represents a DOM Node'''
    content = graphene.String(description='The html representation of the subnodes for the selected DOM',
                              selector=graphene.String())
    html = graphene.String(description='The html representation of the selected DOM',
                           selector=graphene.String())
    text = graphene.String(description='The text for the selected DOM',
                           selector=graphene.String())
    tag = graphene.String(description='The tag for the selected DOM',
                          selector=graphene.String())
    attr = graphene.String(description='The DOM attr of the Node',
                           selector=graphene.String(),
                           name=graphene.String().NonNull)
    _is = graphene.Boolean(description='Returns True if the DOM matches the selector',
                           name='is', selector=graphene.String().NonNull)
    query = graphene.List('Element',
                          description='Find elements using selector traversing down from self',
                          selector=graphene.String().NonNull)
    children = graphene.List('Element',
                             description='The list of children elements from self',
                             selector=graphene.String())
    parents = graphene.List('Element',
                            description='The list of parent elements from self',
                            selector=graphene.String())
    parent = graphene.Field('Element',
                            description='The parent element from self')

    def _query_selector(self, args):
        selector = args.get('selector')
        if not selector:
            return self._root
        return self._root.find(selector)

    def resolve_content(self, args, info):
        return self._query_selector(args).eq(0).html()

    def resolve_html(self, args, info):
        return self._query_selector(args).outerHtml()

    def resolve_text(self, args, info):
        return self._query_selector(args).eq(0).remove('script').text()

    def resolve_tag(self, args, info):
        el = self._query_selector(args).eq(0)
        if el:
            return el[0].tag

    def resolve__is(self, args, info):
        return self._root.is_(args.get('selector'))

    def resolve_attr(self, args, info):
        attr = args.get('name')
        return self._query_selector(args).attr(attr)

    def resolve_query(self, args, info):
        return self._query_selector(args).items()

    def resolve_children(self, args, info):
        selector = args.get('selector')
        return self._root.children(selector).items()

    def resolve_parents(self, args, info):
        selector = args.get('selector')
        return self._root.parents(selector).items()

    def resolve_parent(self, args, info):
        parent = self._root.parents().eq(-1)
        if parent:
            return parent


def get_page(page):
    return pq(page, headers={'user-agent': 'gdom'})


class Document(Node):
    '''
    The Document Type represent any web page loaded and 
    serves as an entry point into the page content
    '''
    title = graphene.String(description='The title of the document')

    def resolve_title(self, args, info):
        return self._root.find('title').eq(0).text()


class Element(Node):
    '''
    A Element Type represents an object in a Document
    '''

    visit = graphene.Field(Document,
                           description='Visit will visit the href of the link and return the corresponding document')

    def resolve_visit(self, args, info):
        # If is a link we follow through href attr
        # return the resulting Document
        if self._root.is_('a'):
            href = self._root.attr('href')
            return get_page(href)


class Query(graphene.ObjectType):
    page = graphene.Field(Document,
                          description='Visit the specified page',
                          url=graphene.String(description='The url of the page'),
                          source=graphene.String(description='The source of the page'))

    def resolve_page(self, args, info):
        url = args.get('url')
        source = args.get('source')
        assert url or source, 'At least you have to provide url or source of the page'
        return get_page(url or source)


schema = graphene.Schema(query=Query)
schema.register(Element)



import argparse
import json
import sys

import flask_graphql
from flask import Flask, Blueprint
from flask_graphql.graphiqlview import GraphiQLView
from flask_graphql.graphqlview import GraphQLView

from schema import schema

SAMPLE_QUERY = '''
{
  page(url:"http://www.yelp.com/biz/amnesia-san-francisco") {
    title: text(selector:"h1")
    phone: text(selector:".biz-phone")
    address: text(selector:".address")
    sections: query(selector:".breadcrumbs--hierarchy a") {
      text
      url: attr(name:"href")
    }
    reviews: query(selector:"[itemprop=review]") {
      date: text(selector:".rating-qualifier")
      rating: attr(selector:"[itemprop=ratingValue]", name:"content")
      username: text(selector:".user-name a")
      comment: text(selector:"p")
    }
  }
}
'''.strip()


def get_test_app():
    app = Flask(__name__)
    app.debug = True

    blueprint = Blueprint('graphql', flask_graphql.__name__,
                          template_folder='templates',
                          static_url_path='/static/graphql',
                          static_folder='static/graphql/')

    app.add_url_rule('/graphql', view_func=GraphQLView.as_view('graphql', schema=schema))
    app.add_url_rule('/', view_func=GraphiQLView.as_view('graphiql', default_query=SAMPLE_QUERY))

    app.register_blueprint(blueprint)

    return app


def parse(query, source, page):
    execution = schema.execute(query, args={'page': page, 'source': source})
    if execution.errors:
        raise Exception(execution.errors[0])
    return execution.data


def main():
    parser = argparse.ArgumentParser(description='Process some integers.')

    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('query', type=argparse.FileType('r'), nargs='?', help='The query file', default=None)
    group.add_argument('--test', action='store_true', default=False, help='This will start a test server with a UI for querying')

    parser.add_argument('page', metavar='PAGE', nargs='?', const=1, type=str, help='The pages to parse')

    parser.add_argument('--source', type=argparse.FileType('r'), default=sys.stdin)
    parser.add_argument('--output', type=argparse.FileType('w'), default=sys.stdout)

    args = parser.parse_args()

    if args.test:
        app = get_test_app()
        import webbrowser
        webbrowser.open('http://localhost:5000/')

        app.run()
    else:
        query = args.query.read()
        page = args.page
        if not sys.stdin.isatty():
            source = args.source.read()
        else:
            source = None
        data = parse(query, source, page)
        outdata = json.dumps(data, indent=4, separators=(',', ': '))
        args.output.write(outdata)
        args.output.write('\n')

if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(name='gdom',
      version='0.1.0',
      download_url='git@github.com:syrusakbary/gdom.git',
      packages=find_packages(),
      author='Syrus Akbary',
      author_email='me@syrusakbary.com',
      description='DOM Traversing and Scraping using GraphQL',
      long_description=open('README.rst').read(),
      keywords='scraping html graphql json',
      url='http://github.com/syrusakbary/gdom',
      license='MIT',
      entry_points={
          'console_scripts': ['gdom = gdom.cmd:main']
      },
      install_requires=[
          'graphene==0.7.3',
          'flask-graphql==1.1.0',
          'pyquery==1.2.11',
          'requests==2.9.1'
      ],
      tests_require=[
      ])
